#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import gi
import subprocess
import utils

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

from pathlib import Path
from gi.repository import Gtk, Gdk, GdkPixbuf     
from os.path import expanduser   


# AÑADIR CLASE --> .get_style_context().add_class("button-none")

#IDIOMAS ----------------------------------------------------------------

# pygettext -d slimbookamdcontrollercopy slimbookamdcontrollercopy.py

currpath = os.path.dirname(os.path.realpath(__file__))

_ = utils.load_translation('slimbookamdcontrollerinfo')

idiomas = utils.get_languages()[0]

user_name = subprocess.getoutput("logname")
user = subprocess.getoutput("echo ~"+user_name)


style_provider = Gtk.CssProvider()
style_provider.load_from_path(currpath+'/css/style.css')

Gtk.StyleContext.add_provider_for_screen (
    Gdk.Screen.get_default(), style_provider,
    Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
)

class PreferencesDialog(Gtk.Dialog):
	
    def __init__(self):
		
        Gtk.Dialog.__init__(self,
			title='',
            parent=None,
            flags=0)

        #Botones de aceptar y cerrar
        #self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT)
        #self.add_buttons(Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT)

        ICON = (currpath+'/images/slimbookamdcontroller.svg')
        #print('Ruta icono: '+ICON)
        
        try: 
            self.set_icon_from_file(ICON)
        except:
            print("Not found")

        self.set_position(Gtk.WindowPosition.CENTER_ALWAYS)     
        self.get_style_context().add_class("bg-color")
        self.set_default_size(900, 0)     
        self.set_decorated(False)

        vbox = Gtk.VBox(spacing=5)
        vbox.set_border_width(5)

        self.get_content_area().add(vbox)

        # Icono APP
        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
			filename= currpath+'/images/logo-sb.png',
			width=200,
			height=100,
			preserve_aspect_ratio=True)
        iconApp = Gtk.Image.new_from_pixbuf(pixbuf)
        iconApp.set_name('top')


        info = Gtk.Label()
        info.set_markup('<span>'+(_("The Slimbook AMD Controller app is capable of setting several TDP power levels for your AMD Ryzen mobile processor. Switching between the different performance presets will give you the ability to control both performance and battery life with a single click. Bear in mind that the higher you set your performance level, your processor will also run hotter and drain your battery faster, so keep that in mind! \n\nSlimbook AMD Controller uses the third party software RyzenAdj from FlyGoat."))+'</span>')
        info.set_line_wrap(True)
        info.set_name('label')
        info.set_name('info')
        
        info2 = Gtk.Label()
        info2.set_markup('<span>'+_("If you want to support the Slimbook team with the development of this app and several more to come, you can do so by joining our ") + "<a href='https://www.patreon.com/slimbook'> patreon </a>" +_(" or buying a brand new Slimbook.")+'</span>')
        info2.set_line_wrap(True)
        info2.set_name('label')
        info2.set_name('info')

        info3 = Gtk.Label()
        info3.set_markup("<span><b>"+_("Note: ")+"</b>"+_("Many laptops limit the power of the CPU, when working without the charger connected. Therefore, if you want to take advantage of the high-performance mode of this application, you may need to connect the charger.")+"</span>")
        info3.set_line_wrap(True)
        info3.set_name('label')
        info3.set_name('info')

        
        enlaces_box = Gtk.Box(spacing=5)
        enlaces_box.set_halign(Gtk.Align.CENTER)

        salvavidas = Gtk.Label(label=_('This software is provided * as is * without warranty of any kind..'))
        salvavidas.set_name('label')

        license1 = Gtk.Label()
        license1.set_markup("<span><b>"+_("You are free from:")+"</b></span>")
        license1.set_name('label')

        license2 = Gtk.Label()
        license2.set_markup("<span><b><small>"+_("Share: ")+"</small></b><small>"+_("copy and redistribute the material in any medium or format\nSlimbook Copyright - License Creative Commons BY-NC-ND")+"</small></span>")
        license2.set_name('label')

        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
			filename=currpath+'/images/cc.png',
			width=100,
			height=50,
			preserve_aspect_ratio=True)

        licencia = Gtk.Image.new_from_pixbuf(pixbuf)

        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
			filename=currpath+'/images/cross.png',
			width=20,
			height=20,
			preserve_aspect_ratio=True)

        close = Gtk.Image.new_from_pixbuf(pixbuf)
        close.set_halign(Gtk.Align.END)

        evnt_close = Gtk.EventBox()
        evnt_close.add(close)
        evnt_close.connect("button_press_event", self.on_button_close)

    # REDES SOCIALES --------------------------------------------------------------
        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
			filename=currpath+'/images/twitter.png',
			width=25,
			height=25,
			preserve_aspect_ratio=True)

        itwitter = Gtk.Image.new_from_pixbuf(pixbuf)
        enlaces_box.pack_start(itwitter, False, False, 0)

        twitter = Gtk.Label()
        twitter.set_markup("<span><b><a href='https://twitter.com/Slimbook'>@Slimbook</a></b>    </span>")
        twitter.set_justify(Gtk.Justification.CENTER)
        
        enlaces_box.pack_start(twitter, False, False, 0)

        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
			filename=currpath+'/images/facebook.png',
			width=25,
			height=25,
			preserve_aspect_ratio=True)

        ifacebook = Gtk.Image.new_from_pixbuf(pixbuf)
        enlaces_box.pack_start(ifacebook, False, False, 0)

        facebook = Gtk.Label()
        facebook.set_markup("<span><b><a href='https://www.facebook.com/slimbook.es'>slimbook.es</a></b>    </span>")
        facebook.set_justify(Gtk.Justification.CENTER)
        enlaces_box.pack_start(facebook, False, False, 0)

        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
			filename=currpath+'/images/insta.png',
			width=25,
			height=25,
			preserve_aspect_ratio=True)

        iinstagram = Gtk.Image.new_from_pixbuf(pixbuf)
        enlaces_box.pack_start(iinstagram, False, False, 0)

        instagram = Gtk.Label()
        instagram.set_markup("<span><b><a href='https://www.instagram.com/slimbookes'>@slimbookes</a></b></span>")
        instagram.set_justify(Gtk.Justification.CENTER)
        enlaces_box.pack_start(instagram, False, False, 0)

    # Enlaces --------------------------------------------------------------------------
        
        link_box2 = Gtk.HBox()
        link_box2.set_halign(Gtk.Align.CENTER)

        #WEB
        web_link=''
        print(idiomas)
        if idiomas.find('es') >= 0:
            web_link = 'https://slimbook.es/es/'
        else:
            web_link = 'https://slimbook.es/en/'

        #TUTORIAL
        tutorial_link=''
        if idiomas.find('es') >= 0:
            tutorial_link = 'https://slimbook.es/es/tutoriales/aplicaciones-slimbook/493-slimbook-amd-controller'
        else:
            tutorial_link = 'https://slimbook.es/en/tutoriales/aplicaciones-slimbook/494-slimbook-amd-controller-en'

        label77 = Gtk.LinkButton(uri=web_link, label =_("Visit @Slimbook web"))
        label77.set_name('link')
        label77.set_halign(Gtk.Align.CENTER)
        link_box2.add(label77)

        label77 = Gtk.LinkButton(uri=tutorial_link, label = (_("@SlimbookAMDController Tutorial")))
        label77.set_name('link')
        label77.set_halign(Gtk.Align.CENTER)
        link_box2.add(label77)

        label77 = Gtk.LinkButton(uri="https://github.com/slimbook/slimbookamdcontroller/tree/main/src/translations", label = (_('Help us with translations!')))
        label77.set_name('link')
        label77.set_halign(Gtk.Align.CENTER)
        link_box2.add(label77)

        email = Gtk.Label()
        email.set_markup("<span><b>"+_("Send an e-mail a: ")+"dev@slimbook.es</b></span>")
        email.set_justify(Gtk.Justification.CENTER)
        email.set_name('label')

        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
            filename= os.path.join(currpath+'/images/', 'GitHub_Logo_White.png'),
            width=150,
            height=30,
            preserve_aspect_ratio=True)

        img = Gtk.Image()
        img.set_from_pixbuf(pixbuf)
        
        github = Gtk.LinkButton(uri="https://github.com/slimbook/slimbookamdcontroller")
        github.set_name('link')
        github.set_halign(Gtk.Align.CENTER)
        github.set_image(img)

    
    # PACKKING ----------------------------------------------------------------------

        vbox.pack_start(evnt_close,True, True, 0)
        vbox.pack_start(iconApp, True, True, 20)

        vbox.pack_start(info, True, True, 20)
        
        vbox.pack_start(enlaces_box, True, True, 5)
        vbox.pack_start(link_box2, True, True, 10)
        vbox.pack_start(github, True, True, 0)
        vbox.pack_start(email, True, True, 10)
        vbox.pack_start(salvavidas, True, True, 0)

        vbox.pack_start(license1, True, True, 0)
        vbox.pack_start(license2, True, True, 0)
        vbox.pack_start(licencia, True, True, 10)
        vbox.pack_start(info2, True, True, 20)
        vbox.pack_start(info3,True, True, 0)

        #SHOW
        self.show_all()
    
    def on_buttonCopyEmail_clicked(self, buttonCopyEmail):
        self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
        self.clipboard.set_text('dev@slimbook.es', -1)
        os.system("notify-send 'Slimbook AMD Controller' "+_("'The email has been copied to the clipboard'") + " -i '" + currpath + "/images/icono.png'")

    def on_button_close(self, button, state):
        self.close()
        self.hide()
        self.destroy()
        Gtk.main_quit







from configparser import ConfigParser

import gi
import os
import re
import utils

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk, Gdk

CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))
APP_NAME= 'slimbookamdcontroller'

config = ConfigParser()
config.read(utils.CONFIG_FILE)

db_cpu = ConfigParser()
db_cpu.read(CURRENT_PATH + '/' + utils.DB_FILE)

_ = utils.load_translation(APP_NAME)

CPU = utils.get_cpu_info('name')

patron = re.compile(r'[ ](.*)[ ]*([0-9]).*([0-9]{4,})(\w*)')
type = patron.search(CPU).group(1).strip()
version = patron.search(CPU).group(2)
number = patron.search(CPU).group(3)
line_suffix = patron.search(CPU).group(4)

MODEL_CPU = type+'-'+version+'-'+number+line_suffix

class WarnDialog(Gtk.Dialog):
    def __init__(self, parent, label):
        super().__init__(title=_("Warning"),transient_for=parent)
        self.set_name('warn')
        self.add_buttons(
            Gtk.STOCK_OK, Gtk.ResponseType.OK
        )

        self.set_default_size(500, 200)

        label.set_line_wrap(True)
        box = self.get_content_area()
        box.add(label)
        self.show_all()

class Settings_dialog(Gtk.Dialog):
    FIELDS = {
                'U':'8000-8000-8000/11000-11000-15000/25000-30000-35000/ 10/25 w',
                'HS':'10000-10000-10000/15000-15000-25000/60000-65000-70000/ 35/35 w',
                'HX':'10000-10000-10000/15000-15000-25000/70000-80000-100000/ 35/45 w',
                'H':'10000-10000-10000/15000-15000-25000/70000-80000-100000/ 35/45 w',
                'G':'20000-20000-20000/25000-25000-35000/70000-75000-80000/ 45/65 w'
            }

    def __init__(self, parent):
        unknown_proc=False
        try:
            default_values = self.FIELDS.get(line_suffix).split('/')
            length = len(default_values)-1
            default_values[length] = default_values[length][:default_values[length].find('w')] 
        except:
            default_values = ['0-0-0', '0-0-0', '0-0-0', '0', '0']
            unknown_proc=True
            print('Creates values.')

        if config.has_option('USER-CPU', 'cpu-parameters') and config['USER-CPU']['cpu-parameters']!= '':
            values = config['USER-CPU']['cpu-parameters'].split('/')
            print('Loads from file.')
        else:
            values = default_values

        self.lowmode = (values[0].split('-'))
        self.midmode = (values[1].split('-'))
        self.highmode = (values[2].split('-'))

        self.low_default = float(default_values[3])

        self.val = '{}/{}/{}/'.format(values[0],values[1],values[2])

        try:
            self.val = self.val+ '{}/{} w'.format(default_values[3], default_values[4])
            self.high_default = float(default_values[4])

        except:
            self.val = self.val+ '{} w'.format(default_values[3])
            self.high_default = float(default_values[3])
            print('Not found 4th value')
        
        print('CPU-Parameters: '+self.val)

        super().__init__(title="Settings", transient_for=parent)
        self.set_name('dialog')
        self.add_buttons(
            Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OK, Gtk.ResponseType.OK
        )
        # WINDOW
        self.set_name('settings')

        center = Gtk.Align.CENTER
        left = Gtk.Align.START
        right = Gtk.Align.END

        win_grid = Gtk.Grid(column_homogeneous=False,
                            row_homogeneous=True,
                            column_spacing=10,
                            row_spacing=10)

        grid = Gtk.Grid(column_homogeneous=False,
                        row_homogeneous=True,
                        column_spacing=20,
                        row_spacing=10)

        self.get_content_area().add(win_grid)

        if self.low_default < 20:
            min = self.low_default - self.low_default/2 
        else:
            min = self.low_default - 3*(self.low_default/4)
        max = self.high_default*2

        # Low TDP Column

        label1 = Gtk.Label(label=_('Low TDP'))

        self.entry01 = _create_gui_element(
            float(self.lowmode[0])/1000, min, max)
        self.entry02 = _create_gui_element(
            float(self.midmode[0])/1000, min, max)
        self.entry03 = _create_gui_element(
            float(self.highmode[0])/1000, min, max)

        grid.attach(self.entry01, 1, 1, 1, 1)
        grid.attach(self.entry02, 1, 2, 1, 1)
        grid.attach(self.entry03, 1, 3, 1, 1)
        grid.attach(label1, 1, 0, 1, 1)


        # Mantained TDP Column

        label1 = Gtk.Label(label=_('Medium TDP'))
        self.entry1 = _create_gui_element(
            float(self.lowmode[1])/1000, min, max)
        self.entry2 = _create_gui_element(
            float(self.midmode[1])/1000, min, max)
        self.entry3 = _create_gui_element(
            float(self.highmode[1])/1000, min, max)

        grid.attach(self.entry1, 2, 1, 1, 1)
        grid.attach(self.entry2, 2, 2, 1, 1)
        grid.attach(self.entry3, 2, 3, 1, 1)
        grid.attach(label1, 2, 0, 1, 1)

        # High Power TDP Column

        label2 = Gtk.Label(label=_('High Power TDP'))

        self.entry4 = _create_gui_element(
            float(self.lowmode[2])/1000, min, max)
        self.entry5 = _create_gui_element(
            float(self.midmode[2])/1000, min, max)
        self.entry6 = _create_gui_element(
            float(self.highmode[2])/1000, min, max)

        grid.attach(self.entry4, 3, 1, 1, 1)
        grid.attach(self.entry5, 3, 2, 1, 1)
        grid.attach(self.entry6, 3, 3, 1, 1)
        grid.attach(label2, 3, 0, 1, 1)

        # Modes Column
        
        low_lbl = Gtk.Label(label=_('Low'), halign=left)
        mid_lbl = Gtk.Label(label=_('Medium'), halign=left)
        high_lbl = Gtk.Label(label=_('High'), halign=left)

        grid.attach(low_lbl, 0, 1, 1, 1)
        grid.attach(mid_lbl, 0, 2, 1, 1)
        grid.attach(high_lbl, 0, 3, 1, 1)

        separator = Gtk.Separator.new(orientation=Gtk.Orientation.HORIZONTAL)
        separator.set_valign(center)
        win_grid.attach(separator, 0, 4, 5, 1)

        recommended_lbl = Gtk.Label(
            label=_('Recommended values:'), halign=left)
        recommended_lbl.set_tooltip_text(_('This values are based on your CPU line suffix'))
        if not unknown_proc:
            low_lbl = Gtk.Label(
            label=_('Low: {} watts'.format(self.low_default)), halign=left)

            high_lbl = Gtk.Label(
            label=_('High: {} watts.'.format(self.high_default)), halign=left)
        else:
            low_lbl = Gtk.Label(
            label=_('Low: {} watts'.format('?')), halign=left)

            high_lbl = Gtk.Label(
            label=_('High: {} watts.'.format('?')), halign=left)

        values_box = Gtk.VBox()
        values_box.add(low_lbl)
        values_box.add(high_lbl)

        win_grid.attach(recommended_lbl, 0, 5, 1, 1)
        win_grid.attach(values_box, 0, 6, 2, 2)
        win_grid.attach(grid, 0, 0, 5, 4)

        self.show_all()

class Dialog(Gtk.Window):
    
    print(MODEL_CPU)
    def __init__(self):
        Gtk.Window.__init__(self, title="Settings")
        
        self.set_icon_from_file(CURRENT_PATH+'/images/slimbookamdcontroller.svg')
        if type.find('Ryzen') != -1:
            if not db_cpu.has_option('PROCESSORS',MODEL_CPU):
                label = Gtk.Label(label=_(
                "Your processor is not supported yet. Do you want to add '{}' to the list?\n"+
                "If this is the case, you should go to the processor's vendor page and get information about your TDP values.").format(CPU))
                
                dialog = WarnDialog(self, label)

                response = dialog.run()

                dialog.close()
                dialog.destroy()
                    
            dialog = Settings_dialog(self)
            dialog.show_all()
            response = dialog.run()

            if response == Gtk.ResponseType.OK:
                self.accept(dialog)
                dialog.close()
                dialog.destroy()
                
            elif response == Gtk.ResponseType.CANCEL:
                dialog.close()
                dialog.destroy()
            
            
        else:
            label = Gtk.Label(label=_('Any AMD Ryzen processor detected!'))
            dialog = WarnDialog(self, label)
            response = dialog.run()

        dialog.close()
        dialog.destroy()

    def accept(self, dialog):
        new_values = '{}-{}-{}/{}-{}-{}/{}-{}-{}/  {}/{}'.format(
                                           int(dialog.entry01.get_text())*1000,
                                           int(dialog.entry1.get_text())*1000,
                                           int(dialog.entry4.get_text())*1000,
                                           int(dialog.entry02.get_text())*1000,
                                           int(dialog.entry2.get_text())*1000,
                                           int(dialog.entry5.get_text())*1000,
                                           int(dialog.entry03.get_text())*1000,
                                           int(dialog.entry3.get_text())*1000,
                                           int(dialog.entry6.get_text())*1000,
                                           dialog.low_default, dialog.high_default)

        config.set('USER-CPU','cpu-parameters', new_values)
        """
        if config.has_option('PROCESSORS',MODEL_CPU):
            print('Processor values modified.')
            config.set('USER-CPU','cpu-parameters', new_values)

        else: 
            print('Processor added')
            if not config.has_section('PROCESSORS'):
                config.add_section('PROCESSORS')
            config.set('PROCESSORS', MODEL_CPU, dialog.val)
            if not config.has_section('USER-CPU'):
                config.add_section('USER-CPU')
            config.set('USER-CPU','cpu-parameters', new_values)
            print(config['PROCESSORS'][MODEL_CPU])
        """
        with open(utils.CONFIG_FILE, 'w') as configfile:
                config.write(configfile)

def _create_gui_element(value, low, max):

    spin_button = Gtk.SpinButton()

    if max != 0:
        spin_button.set_adjustment(Gtk.Adjustment(
            value=value,
            lower=low,
            upper=max,
            step_increment=1,
            page_increment=10,
        ))
    else:
        spin_button.set_adjustment(Gtk.Adjustment(
            value=value,
            lower=low,
            upper=float(100),
            step_increment=1,
            page_increment=10,
        ))

    spin_button.set_numeric(True)

    return spin_button

style_provider = Gtk.CssProvider()
style_provider.load_from_path(CURRENT_PATH+'/css/style.css')

Gtk.StyleContext.add_provider_for_screen(
    Gdk.Screen.get_default(), style_provider,
    Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
)
if __name__ == "__main__":
    Dialog()
    Gtk.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# This file checks if any new configuration has been added an then, adds it to old configuration file
# in order to keep old custom values.
# It also gives permisions to config directory

import os, pwd
import sys, logging
import subprocess
import configparser

APPNAME = 'slimbookamdcontroller'

USERNAME = subprocess.getstatusoutput("logname")
# print(str(USERNAME))

# 1. Try getting logged username  2. This user is not root  3. Check user exists (no 'reboot' user exists)
if USERNAME[0] == 0 and USERNAME[1] != 'root' and subprocess.getstatusoutput('getent passwd '+USERNAME[1])[0] == 0:
    print('Getting logname successfull')
    USER_NAME = USERNAME[1]
else:
    print('Getting logname ...')
    USER_NAME = subprocess.getoutput('last -wn1 | head -n 1 | cut -f 1 -d " "')
print('Config check executed as '+USER_NAME)

HOMEDIR = subprocess.getoutput("echo ~"+USER_NAME)

CURRPATH = os.path.dirname(os.path.realpath(__file__))

uid, gid = pwd.getpwnam(USER_NAME).pw_uid, pwd.getpwnam(USER_NAME).pw_gid

# There will always be a folder in src called configuration, where default configuration is stored.
# There will also be a route in
DEFAULT_CONF = CURRPATH+'/'+APPNAME+'.conf'
CONFIG_FILE = HOMEDIR+'/.config/'+APPNAME+'/'+APPNAME+'.conf'
CONFIG_FOLDER = HOMEDIR+'/.config/'+APPNAME
logger = logging.getLogger()

def main(args):

    check1()

    if subprocess.getoutput('whoami') == 'root':
        print('Giving permisions to config directory...')
        os.system('sudo chmod 777 -R '+HOMEDIR+'/.config/'+APPNAME)
        
    set_ownership(CONFIG_FOLDER)

def set_ownership(folder):
    folder_stat = os.stat(folder)
    f_uid = folder_stat.st_uid
    f_gid = folder_stat.st_gid
    logger.debug("Folder {}\nUser uid={}\nFolder uid={}".format(folder, uid, f_gid))

    if not uid == f_uid or not gid == f_gid:
        #logger.info('Setting {} ownership').format(folder)

        for dir_path, dir_name, filenames in os.walk(folder):
            logger.debug(dir_path)
            os.chown(dir_path, uid, gid)
            for filename in filenames:
                file_path = os.path.join(dir_path, filename)
                logger.debug(file_path)
                os.chown(file_path, uid, gid)

def check1():
    if os.path.isfile(CONFIG_FILE):
        config = configparser.ConfigParser()
        config.read(CONFIG_FILE)

        vars = subprocess.getoutput('cat '+DEFAULT_CONF).split('\n')
        # print(str(vars))
        incidences = False

        section = ''
        for var in vars:
            PROCS = {'U','H','HS','HX','G'}
            if var.find('=') != -1:
                # We get the line, then the variable name, then we remove spaces at the start or the end
                value = var.split('=')[1].strip()
                var = var.split('=')[0].strip()
                try:
                    config[section][var]
                    if var in PROCS :
                        # if new value different setting defaults 

                        oldvalue = config.get(section, var)
                        if oldvalue != value:
                            incidences = True
                            print('Updating default values and '+var)
                            config.set('USER-CPU', 'cpu-parameters', '')
                            config.set('PROCESSORS', var, value)

                except:
                    incidences = True
                    print('Not found: '+var)
                    try:
                        config.set(section, var, value)
                    except:
                        config.add_section(section)
                        print('Section added')
                        config.set(section, var, value)

            else:
                if var.startswith('[') and var.endswith(']'):
                    section = var[1:len(var)-1]
                    print('Checking section: '+section+'...')

        if incidences:
            try:
                configfile = open(CONFIG_FILE, 'w')
                config.write(configfile)
                configfile.close()
                print('Incidences corrected.')
            except:
                print('Incidences could not be corrected.')
        else:
            print('Incidences not found.')
    else:
        print('Creating config file ...')
        os.system('mkdir -p '+HOMEDIR+'/.config/'+APPNAME +
                  '/ && cp '+DEFAULT_CONF+' '+CONFIG_FILE)

if __name__ == "__main__":
    logger.setLevel(logging.DEBUG)
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.DEBUG)
    #formatter = logging.Formatter('%(asctime)s - %(funcName)s:%(lineno)d - %(levelname)s - %(message)s')
    formatter = logging.Formatter('%(asctime)s - %(lineno)d - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

main(sys.argv)



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from configparser import ConfigParser
import gi, os, subprocess, logging, sys, signal
import shutil, math, re
from sections.gpu_section import GpuSection
import utils
from pathlib import Path
import slimbookamdcontrollerinfo as info

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')

from gi.repository import Gdk, Gtk, GLib, GdkPixbuf

APPNAME = 'slimbookamdcontroller'
HOMEDIR = str(Path.home())

CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))
BASE_DIR = os.path.dirname(os.path.abspath(__file__))

LAUNCHER_DESKTOP = os.path.join(
    BASE_DIR, "{}-autostart.desktop".format(APPNAME))
AUTOSTART_DESKTOP = "{}/.config/autostart/{}-autostart.desktop".format(HOMEDIR, APPNAME)

_ = utils.load_translation(APPNAME)

iconpath = CURRENT_PATH+'/amd.png'

cpu = utils.get_cpu_info('name')
patron = re.compile(r'[ ](.*)[ ]*([0-9]).*([0-9]{4,})(\w*)')
type = patron.search(cpu).group(1).strip()
gen = patron.search(cpu).group(2)
number = patron.search(cpu).group(3)
line_suffix = patron.search(cpu).group(4)

config = ConfigParser()
config.read(utils.CONFIG_FILE)

db_cpu = ConfigParser()
db_cpu.read(CURRENT_PATH + '/' + utils.DB_FILE)

class SlimbookAMD(Gtk.ApplicationWindow):

    mode = ""
    indicador_actual = ""
    autostart_actual = ""
    parameters = ''
    cpu_ok = False

    switch1 = Gtk.Switch()
    switch1.set_halign(Gtk.Align.END)

    switch2 = Gtk.Switch()
    switch2.set_halign(Gtk.Align.END)

    rbutton1 = Gtk.RadioButton.new_with_label_from_widget(None, (_("Low")))
    rbutton2 = Gtk.RadioButton.new_with_mnemonic_from_widget(rbutton1, (_("Medium")))
    rbutton3 = Gtk.RadioButton.new_with_mnemonic_from_widget(rbutton1, (_("High")))

    def __init__(self):
        Gtk.Window.__init__(self, title="Slimbook AMD Controller")

        self.set_icon()
        self.inicio()

        #self.set_decorated(False)

        self.set_position(Gtk.WindowPosition.CENTER)
        self.get_style_context().add_class("bg-image")

        # Movement
        self.active = True
        self.is_in_drag = False
        self.x_in_drag = 0
        self.y_in_drag = 0
        self.connect('button-press-event', self.on_mouse_button_pressed)
        self.connect('button-release-event', self.on_mouse_button_released)
        self.connect('motion-notify-event', self.on_mouse_moved)

        self.win_grid = Gtk.Grid(column_homogeneous=True,
                                 column_spacing=0,
                                 row_spacing=10)

        self.add(self.win_grid)

        #Content
        
        label1 = Gtk.Label(label=_("Enable app at startup"))
        label1.set_halign(Gtk.Align.START)

        label2 = Gtk.Label(label=_("Show indicator icon"))
        label2.set_halign(Gtk.Align.START)

        button1 = Gtk.Button(label="Button 1")
        button1.set_halign(Gtk.Align.START)
        button1.get_style_context().add_class("button-none")

        button2 = Gtk.Button(label="Button 2")
        button2.set_halign(Gtk.Align.START)
        button2.get_style_context().add_class("button-none")

        separador = Gtk.Image.new_from_file(CURRENT_PATH+'/images/separador.png')
        separador.set_halign(Gtk.Align.CENTER)
        separador2 = Gtk.Image.new_from_file(CURRENT_PATH+'/images/separador.png')
        separador2.set_halign(Gtk.Align.CENTER)
        separador3 = Gtk.Image.new_from_file(CURRENT_PATH+'/images/separador.png')
        separador3.set_halign(Gtk.Align.CENTER)

        pixbuf1 = GdkPixbuf.Pixbuf.new_from_file_at_scale(
            filename=CURRENT_PATH+'/images/cross.png',
            width=20,
            height=20,
            preserve_aspect_ratio=True)

        close = Gtk.Image.new_from_pixbuf(pixbuf1)
        close.get_style_context().add_class("close")

        evnt_close = Gtk.EventBox()
        evnt_close.add(close)
        evnt_close.set_halign(Gtk.Align.END)
        evnt_close.connect("button_press_event", self.on_btnCerrar_clicked)

        # CPU
        cpuGrid = Gtk.Grid(column_homogeneous=True,
                           column_spacing=0,
                           row_spacing=12)
        cpuGrid.set_name('cpuGrid')
        cpuGrid.set_valign(Gtk.Align.CENTER)
        
        # CPU Name
        hbox_consumo = Gtk.HBox(spacing=15)

        cpu_name = Gtk.Label(label=cpu)
        cpu_name.set_halign(Gtk.Align.CENTER)
        hbox_consumo.pack_start(cpu_name, True, True, 0)

        # CPU Temp
        thermal_zones = subprocess.getstatusoutput(
            'ls /sys/class/thermal/ | grep thermal_zone')[1].split('\n')

        cpu_thermal_zone = None
        for thermal_zone in thermal_zones:
            if subprocess.getstatusoutput("cat /sys/class/thermal/"+thermal_zone+"/type | grep acpitz")[0] == 0:
                print('Found thermal zone!')
                cpu_thermal_zone = thermal_zone
                exit

        if cpu_thermal_zone != None:
            cpu_temp = subprocess.getstatusoutput(
                "cat /sys/class/thermal/"+cpu_thermal_zone+r"/temp | sed 's/\(.\)..$/ °C/'")
            if cpu_temp[0] == 0:
                label = Gtk.Label(label=cpu_temp[1])

                def _update_label(label: Gtk.Label):
                    label.set_label(subprocess.getoutput(
                        "cat /sys/class/thermal/"+cpu_thermal_zone+r"/temp | sed 's/\(.\)..$/ °C/'"))
                    return True

                GLib.timeout_add_seconds(2, _update_label, label)
                hbox_consumo.pack_start(label, True, True, 0)
            else:
                print('Cpu_temp 404')
        else:
            print('Thermal_zone 404')

        # consumo = Gtk.Label(label=_('Current consumption: ')+ self.cpu_value('slow-limit')+" - "+self.cpu_value('stapm-limit')+" - "+self.cpu_value('fast-limit')+" mW.")
        # consumo.set_halign(Gtk.Align.END)
        # hbox_consumo.pack_start(consumo, True, True, 0)

        img = ''

        idiomas = utils.get_languages()[0]

        print (idiomas)
        if idiomas.find("es") >= 0:  # Español
            img = 'modos_es.png'
        else:
            img = 'modos_en.png'  # Inglés

        modos = Gtk.Image.new_from_file(CURRENT_PATH+'/images/'+img)
        modos.get_style_context().add_class("cambModos")
        modos.set_halign(Gtk.Align.CENTER)

        # Box 1
        vbox1 = Gtk.VBox()
        self.rbutton1.connect("toggled", self.on_button_toggled, "low")
        self.rbutton1.set_name('radiobutton')
        self.rbutton1.set_halign(Gtk.Align.CENTER)

        rbutton1_img = Gtk.Image.new_from_file(CURRENT_PATH+'/images/modo1.png')
        rbutton1_img.set_halign(Gtk.Align.CENTER)

        vbox1.pack_start(rbutton1_img, False, False, 0)
        vbox1.pack_start(self.rbutton1, False, False, 0)

        # Box 2
        vbox2 = Gtk.VBox()
        self.rbutton2.connect("toggled", self.on_button_toggled, "medium")
        self.rbutton2.set_name('radiobutton')
        self.rbutton2.set_halign(Gtk.Align.CENTER)

        rbutton2_img = Gtk.Image.new_from_file(CURRENT_PATH+'/images/modo2.png')
        rbutton2_img.set_halign(Gtk.Align.CENTER)

        vbox2.pack_start(rbutton2_img, False, False, 0)
        vbox2.pack_start(self.rbutton2, False, False, 0)

        # Box 3
        vbox3 = Gtk.VBox()
        self.rbutton3.connect("toggled", self.on_button_toggled, "high")
        self.rbutton3.set_name('radiobutton')
        self.rbutton3.set_halign(Gtk.Align.CENTER)

        rbutton3_img = Gtk.Image.new_from_file(CURRENT_PATH+'/images/modo3.png')
        rbutton3_img.set_halign(Gtk.Align.CENTER)

        vbox3.pack_start(rbutton3_img, False, False, 0)
        vbox3.pack_start(self.rbutton3, False, False, 0)

        hbox_radios = Gtk.HBox(spacing=70)
        hbox_radios.set_valign(Gtk.Align.CENTER)
        hbox_radios.add(vbox1)
        hbox_radios.add(vbox2)
        hbox_radios.add(vbox3)

        cpuGrid.attach(hbox_consumo, 4, 8, 5, 1)
        cpuGrid.attach(modos, 5, 10, 3, 1)
        cpuGrid.attach(hbox_radios, 4, 11, 5, 3)

        # Buttons
        botonesBox = Gtk.Box(spacing=10)
        botonesBox.set_name('botonesBox')
        botonesBox.set_halign(Gtk.Align.CENTER)
        botonesBox.set_name('buttons')

        # Accept
        btnAceptar = Gtk.ToggleButton(label=_("ACCEPT"))
        btnAceptar.set_size_request(125, 30)
        btnAceptar.connect("toggled", self.on_btnAceptar_clicked)
        botonesBox.pack_start(btnAceptar, True, True, 0)

        # Close
        btnCancelar = Gtk.ToggleButton(label=_("CANCEL"))
        btnCancelar.set_size_request(125, 30)
        btnCancelar.connect("toggled", self.on_btnCerrar_clicked, 'x')
        botonesBox.pack_start(btnCancelar, True, True, 0)

        # Notebook
        notebook = Gtk.Notebook()

        page1 = Gtk.Box()
        page1.set_orientation(Gtk.Orientation.HORIZONTAL)
        page1.set_border_width(0)
        page1.set_halign(Gtk.Align.CENTER)
        page1.add(cpuGrid)
        notebook.append_page(page1, Gtk.Label(label="CPU"))

        # GPU
        notebook = GpuSection(notebook).add()

        # About us 
        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
            filename=CURRENT_PATH+'/images/question.png',
            width=23,
            height=23,
            preserve_aspect_ratio=True)

        iconApp = Gtk.Image.new_from_pixbuf(pixbuf)
        iconApp.get_style_context().add_class("help")

        evnt_box = Gtk.EventBox()
        evnt_box.set_halign(Gtk.Align.END)
        evnt_box.add(iconApp)

        evnt_box.connect("button_press_event", self.about_us)

        pixbuf1 = GdkPixbuf.Pixbuf.new_from_file_at_scale(
            filename = CURRENT_PATH+'/images/settings.png',
            width = 20,
            height = 20,
            preserve_aspect_ratio=True)

        settings = Gtk.Image.new_from_pixbuf(pixbuf1)
        evnt_settings = Gtk.EventBox()
        evnt_settings.set_valign(Gtk.Align.CENTER)
        evnt_settings.set_halign(Gtk.Align.END)
        evnt_settings.add(settings)
        evnt_settings.connect("button_press_event", self.settings)

        hbox_close = Gtk.HBox()
        hbox_close.set_valign(Gtk.Align.START)
        hbox_close.set_halign(Gtk.Align.END)
        hbox_close.add(evnt_settings)
        hbox_close.add(evnt_close)

        version_tag = Gtk.Label(label='')
        version_tag.set_halign(Gtk.Align.START)
        version_tag.set_valign(Gtk.Align.END)
        version_tag.set_name('version')
        version_line = subprocess.getstatusoutput(
            "cat "+CURRENT_PATH+r"/changelog |head -n1| egrep -o '\(.*\)'")
        if version_line[0] == 0:
            version = version_line[1]
            version_tag.set_markup(
                '<span font="10">Version '+version[1:len(version)-1]+'</span>')
        version_tag.set_justify(Gtk.Justification.CENTER)

        # Grid 
        grid = Gtk.Grid(column_homogeneous=False,
                        column_spacing=0,
                        row_spacing=10)

        grid.set_name('label')

        grid.attach(button1, 1, 1, 2, 1)
        grid.attach(button2, 9, 1, 2, 1)

        grid.attach(label1, 4, 4, 3, 1)
        grid.attach(self.switch1, 6, 4, 2, 1)
        grid.attach(separador, 2, 5, 8, 1)
        grid.attach(label2, 4, 6, 3, 1)
        grid.attach(self.switch2, 6, 6, 2, 1)
        grid.attach(separador2, 2, 7, 8, 1)

        """ grid.attach(hbox_consumo, 2, 8, 8, 1)
        grid.attach(modos, 2, 10, 8, 1)
        grid.attach(hbox_radios, 3, 11, 6, 2) """

        grid.attach(notebook, 2, 8, 8, 1)

        self.win_grid.attach(grid, 1, 1, 5, 5)
        self.win_grid.attach(botonesBox, 1, 7, 5, 1)
        self.win_grid.attach(version_tag, 1, 7, 5, 1)
        self.win_grid.attach(evnt_box, 5, 7, 1, 1)
        self.win_grid.attach(hbox_close, 5, 0, 1, 1)
        self.show_all()

        self.set_cpu()

        try:
            params = config.get('USER-CPU', 'cpu-parameters').split('/')
            self.parameters = params
        except:
            print('CPU not added')


    def settings(self, widget=None, x=None):
        import settings
        self.active = False
        dialog = settings.Dialog()

    def set_icon(self):
        if str(Path(__file__).parent.absolute()).startswith('/usr'):
            SHAREDIR = os.path.join('/usr', 'share')
            ICONDIR = os.path.join(
                SHAREDIR, 'icons', 'hicolor', 'scalable', 'apps')
        else:
            ROOTDIR = os.path.dirname(__file__)
            ICONDIR = os.path.normpath(
                os.path.join(ROOTDIR, 'images'))

        ICON = os.path.join(ICONDIR, 'slimbookamdcontroller.svg')

        try:
            self.set_icon_from_file(ICON)
        except:
            print("Icon not found")

    def on_realize(self, widget):
        monitor = Gdk.Display.get_primary_monitor(Gdk.Display.get_default())
        scale = monitor.get_scale_factor()
        monitor_width = monitor.get_geometry().width / scale
        monitor_height = monitor.get_geometry().height / scale
        width = self.get_preferred_width()[0]
        height = self.get_preferred_height()[0]
        self.move((monitor_width - width)/2, (monitor_height - height)/2)

    def on_mouse_moved(self, widget, event):
        if self.is_in_drag:
            xi, yi = self.get_position()
            xf = int(xi + event.x_root - self.x_in_drag)
            yf = int(yi + event.y_root - self.y_in_drag)
            if math.sqrt(math.pow(xf-xi, 2) + math.pow(yf-yi, 2)) > 10:
                self.x_in_drag = event.x_root
                self.y_in_drag = event.y_root
                self.move(xf, yf)
        return False
        
    def on_mouse_button_released(self, widget, event):
        if event.button == 1:
            self.is_in_drag = False
            self.x_in_drag = event.x_root
            self.y_in_drag = event.y_root
        return False
        
    def on_mouse_button_pressed(self, widget, event):

        if event.button == 1 and self.active == True:
            self.is_in_drag = True
            self.x_in_drag, self.y_in_drag = self.get_position()
            self.x_in_drag = event.x_root
            self.y_in_drag = event.y_root
            #return True
        return False

    def on_button_toggled(self, button, name):
        self.mode = name

    def on_btnAceptar_clicked(self, widget):

        # Check secureboot

        if utils.get_secureboot_status():
            self.dialog(_("Secureboot Warning"),
                        _("This computer has Secureboot enabled, which does not allow kernel to manage CPU parameters."))
            sys.exit(1)

        modes = {"low":0, "medium":1, "high":2}
        nmode = modes[self.mode]

        try:
            # This throws an exception if cpu is not found
            set_parameters = self.parameters[nmode].split('-')
            print(set_parameters)

            print('Updating '+self.mode+' to : ' +
                  set_parameters[0]+' '+set_parameters[1]+' '+set_parameters[2]+'.\n')

            returncode = subprocess.call('pkexec slimbookamdcontroller-pkexec {}'.format(utils.CONFIG_FILE), shell=True, stdout=subprocess.PIPE)
            print("return code:",returncode)
            if (returncode == 0):
                os.system("notify-send 'Slimbook AMD Controller' '" + _("Changes have been executed correctly.") +                          "' -i '" + CURRENT_PATH+'/images/slimbookamdcontroller.svg' + "'")
                

                # Check switches
                self._inicio_automatico(self.switch1, self.switch1.get_state())

                self._show_indicator(self.switch2, self.switch2.get_state())

                # Update vars
                self.update_config_file("mode", self.mode)
                self.update_config_file("autostart", self.autostart_actual)
                self.update_config_file("show-icon", self.indicador_actual)

                #self.reboot_indicator()

            else:
                os.system("notify-send 'Slimbook AMD Controller' '" + _("Your CPU is not avalible, this software might not work.") +
                          "' -i '" + CURRENT_PATH+'/images/slimbookamdcontroller.png' + "'")
        except Exception as e:
            print(e)
            os.system("notify-send 'Slimbook AMD Controller' '" + _("Your CPU is not avalible, this software might not work.") +
                      "' -i '" + CURRENT_PATH+'/images/slimbookamdcontroller.png' + "'")

        # CERRAMOS PROGRAMA
        Gtk.main_quit()

    def dialog(self, title, message, link=None):
        dialog = Gtk.MessageDialog(
            transient_for=self,
            flags=0,
            message_type=Gtk.MessageType.WARNING,
            buttons=Gtk.ButtonsType.CLOSE,
            text=title,
        )

        dialog.format_secondary_text(
            message
        )

        dialog.set_position(Gtk.WindowPosition.CENTER)

        response = dialog.run()

        if response == Gtk.ResponseType.CLOSE:

            print("WARN dialog closed")

        dialog.destroy()

    def inicio(self):
        print('Loading configuration:\n')
        
        if not Path(utils.CONFIG_FILE).exists():
            os.makedirs(os.path.expanduser("~/.config/slimbookamdcontroller"),exist_ok = True)
            self.update_config_file("autostart","off","CONFIGURATION")
            self.update_config_file("show-icon","off","CONFIGURATION")
            self.update_config_file("mode","medium","CONFIGURATION")
        
        config.read(utils.CONFIG_FILE)
        
        if config.get('CONFIGURATION', 'autostart') == 'on':
            self.autostart_actual = 'on'
            self.switch1.set_active(True)
            print('- Autostart enabled')
        else:
            self.autostart_actual = 'off'
            self.switch1.set_active(False)
            print('- Autostart disabled')

        if config.get('CONFIGURATION', 'show-icon') == 'on':
            self.indicador_actual = 'on'
            self.switch2.set_active(True)
            print('- Indicator enabled')
        else:
            self.indicador_actual = 'off'
            self.switch2.set_active(False)
            print('- Indicator disabled')

        self.mode = config.get('CONFIGURATION', 'mode')
        
        self.rbutton1.set_active(self.mode == "low")
        self.rbutton2.set_active(self.mode == "medium")
        self.rbutton3.set_active(self.mode == "high")
        

    # Read RyzenAdj output
    def cpu_value(self, parameter):

        call = subprocess.getoutput(
            '/usr/bin/ryzenadj --info')
        salida = str(call)

        index = str(salida.find(parameter))
        #print('Indice: '+index)

        # Que encuentre cuatro digitos juntos seguidos de 1 o 2 letras mayusculas
        patron = re.compile(r"([0-9]{1,2}\.[0-9]{3,})[ ]\|[ ]("+parameter+")")
        value = patron.search(call).group(1)
        param = patron.search(call).group(2)

        print('Value: '+str(value))
        print('Param: '+param)

        return value

    def reboot_indicator(self):
        pid = utils.get_pid_from_file("slimbook.amd.controller.indicator.pid")

        if (pid>0):
            if (utils.is_pid_alive(pid)):
                print("Killing indicator process...")
                os.kill(pid,9)

        if self.indicador_actual:
            print("Starting indicator...")
            os.system("python3 " + CURRENT_PATH + "/slimbookamdcontrollerindicator.py")

    # Copies autostart file in directory
    def _inicio_automatico(self, switch, state):

        if not os.path.exists(HOMEDIR+'/.config/autostart'):
            os.system('mkdir ~/.config/autostart')
            print('Dir autostart created.')

        if switch.get_active() is True:
            print("\nAutostart Enabled")
            if not os.path.isfile(AUTOSTART_DESKTOP):
                shutil.copy(LAUNCHER_DESKTOP, AUTOSTART_DESKTOP)
                print("File .conf has been copied!.")
            self.autostart_actual = 'on'
        else:
            print("\nAutostart Disabled")
            if os.path.isfile(AUTOSTART_DESKTOP):
                os.remove(AUTOSTART_DESKTOP)
                print("File .conf has been deleted.")
            self.autostart_actual = 'off'

        print('Autostart now: ' + self.autostart_actual+'')

    def set_cpu(self):
        if not config.has_option('USER-CPU', 'cpu-parameters') or len(config.get('USER-CPU', 'cpu-parameters')) <= 1:
            print('Setting cpu TDP values')
            cpu_codename = type+'-'+gen+'-'+number+line_suffix

            params = db_cpu['PROCESSORS'][cpu_codename]
            self.update_config_file('cpu-parameters', params, 'USER-CPU')
            
            """
            if config.has_option('PROCESSORS',cpu_codename):
                print('Found processor in list')
                params = config['PROCESSORS'][cpu_codename]
                self.update_config_file('cpu-parameters', params, 'USER-CPU')
            else:
                print('Could not find your proc in .conf')
                self.settings()
                config.read(utils.CONFIG_FILE)
            """

    def _show_indicator(self, switch, state):

        if switch.get_active() is True:
            print("\nIndicator Enabled")
            # --> Luego esta variable será guardada y cargada desde el programa indicador
            self.indicador_actual = 'on'
        else:
            print("\nIndicator Disabled")
            self.indicador_actual = 'off'

        print('Indicador now: ' + self.indicador_actual)
        print()

    def about_us(self, widget, x):
        self.active = False
        print('\nINFO:')
        #os.system('sudo /usr/share/slimbookamdcontroller/ryzenadj --info')
        print('\n')
        # Abre la ventana de info

        dialog = info.PreferencesDialog()
        dialog.connect("destroy", self.close_dialog)
        dialog.show_all()

        #os.system('python3 '+currpath+'/slimbookamdcontrollerinfo.py')

    def close_dialog(self, dialog):
        dialog.close()

        self.active = True

    def on_btnCerrar_clicked(self, widget, x):
        Gtk.main_quit()

    def update_config_file(self, variable, value, section='CONFIGURATION'):

        config.read(utils.CONFIG_FILE)

        # We change our variable: config.set(section, variable, value)
        if not config.has_section(section):
            config.add_section(section)
        
        config.set(section, str(variable), str(value))

        # Writing our configuration file
        with open(utils.CONFIG_FILE, 'w') as configfile:
            config.write(configfile)

        print("\n- Variable |"+variable +
              "| updated in .conf, actual value: "+value)

def main():
    pid_name = "slimbook.amd.controller.application.pid"
    utils.application_lock(pid_name)
    
    win = SlimbookAMD()
    win.connect("destroy", Gtk.main_quit)
    Gtk.main()
    
    utils.application_release(pid_name)
    
    sys.exit(0)

if __name__=="__main__":
    main()



MODEL = 'Model'
VRAM = 'VRAM'
VRAM_USAGE = 'VRAM Usage'
TEMP = 'Temp'
GPU_FREQ = 'GPU Freq'
MEM_FREQ = 'Mem Freq'
PCI_SLOT = 'PCI Slot'
GPU_VOLT = 'GPU Voltage'

DYNAMIC_GPU_PROPERTIES = [TEMP, GPU_FREQ, MEM_FREQ, VRAM_USAGE, GPU_VOLT]

UNKNOWN_MODEL = 'Unknown model'

REGEX_MODEL_WITH_MODEL_IN_BRACKETS_MATCH = r"Advanced\sMicro\sDevices,\sInc\..\[AMD\/ATI\]\s[a-zA-Z0-9\/\s]+\s\[[a-zA-Z0-9\s\/]+\]"
REGEX_MODEL_WITH_MODEL_IN_BRACKETS_FINDALL = r"\s\[[a-zA-Z0-9\s\/]+\]"

REGEX_MODEL_ONLY_SERIES_NAME_MATCH = r"Advanced\sMicro\sDevices,\sInc\..\[AMD\/ATI\]\s[a-zA-Z0-9\s\/]+\s\([a-zA-Z0-9\s\/]+\)"
REGEX_MODEL_ONLY_SERIES_NAME_FINDALL = r"\s[a-zA-Z0-9\s\/]+\s"


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#

import signal
import subprocess
import os
import gi
import sys
from pathlib import Path

gi.require_version('AyatanaAppIndicator3', '0.1')
from gi.repository import AyatanaAppIndicator3 as AppIndicator3
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from configparser import ConfigParser

# We want load first current location
CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))
if CURRENT_PATH not in sys.path:
    sys.path = [CURRENT_PATH] + sys.path
import utils

srcpath = '/usr/share/slimbookamdcontroller/src'
sys.path.insert(1, srcpath)

CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))

#Variables
_ = utils.load_translation('slimbookamdcontrollerindicator')
ICON_FILE = CURRENT_PATH+'/amd.png'

config = ConfigParser()

#Menu
class Indicator():
    modo_actual="none"
    parameters=('','','')
    icono_actual = ICON_FILE

    def __init__(self):
        self.app = 'com.slimbook.amdcontroller'
        iconpath = CURRENT_PATH+'/amd.png'
        # after you defined the initial indicator, you can alter the icon!
        self.testindicator = AppIndicator3.Indicator.new(
            self.app, iconpath, AppIndicator3.IndicatorCategory.HARDWARE)
        self.testindicator.set_icon_theme_path(os.path.join(CURRENT_PATH, 'images'))
        self.testindicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
        self.testindicator.set_menu(self.create_menu())
        self.inicio()

    def create_menu(self):
        menu = Gtk.Menu()

        icon_bajo = Gtk.Image()
        icon_bajo.set_from_file(CURRENT_PATH+'/images/amd-1.png')

        icon_medio = Gtk.Image()
        icon_medio.set_from_file(CURRENT_PATH+'/images/amd-2.png')

        icon_alto = Gtk.Image()
        icon_alto.set_from_file(CURRENT_PATH+'/images/amd-3.png')


        item_bajo = Gtk.ImageMenuItem(label=_('Low performance'), image = icon_bajo)
        item_bajo.connect('activate', self.bajorendimiento)
        #item_bajo.set_always_show_image(True)

        item_medio = Gtk.ImageMenuItem(label=_('Medium performance'), image=icon_medio)
        item_medio.connect('activate', self.mediorendimiento)

        #item_medio.set_always_show_image(True)

        item_alto = Gtk.ImageMenuItem(label=_('High performance'), image= icon_alto)
        item_alto.connect('activate', self.altorendimiento)
        #item_alto.set_always_show_image(True)

        item_ventana = Gtk.MenuItem(label=_('Preferences'))
        item_ventana.connect('activate', self.openWindow)

        item_separador = Gtk.SeparatorMenuItem()

        item_quit = Gtk.MenuItem(label=_('Exit'))
        item_quit.connect('activate', self.exit)

        menu.append(item_bajo)
        menu.append(item_medio)
        menu.append(item_alto)
        menu.append(item_separador)
        menu.append(item_ventana)
        menu.append(item_quit)
        menu.show_all()

        return menu

    def exit(self, source):
        Gtk.main_quit()

    def openWindow(self, source):
        subprocess.Popen('slimbookamdcontroller', shell = True)

    def inicio(self):
        config = ConfigParser()
        config.read(utils.CONFIG_FILE)

        print('Loading data from .conf:\n')

        #Mostramos indicador, o no :):
        if config.get('CONFIGURATION', 'show-icon') == 'on':
            self.testindicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
            print('- Autostart enabled')
        else:
            self.testindicator.set_status(AppIndicator3.IndicatorStatus.PASSIVE)
            print('- Autostart disabled')


        #Cargamos los parametros de la CPU
        params = config.get('USER-CPU', 'cpu-parameters').split('/')
        self.parameters = params

        print('- CPU Parameters: {}'.format(str(self.parameters)))


        if config.get('CONFIGURATION', 'mode') == "low":
            print('- Low\n')
            self.bajorendimiento()
        else:
            if config.get('CONFIGURATION', 'mode') == "medium":
                print('- Medium\n')
                self.mediorendimiento()
            else:
                print('- High\n')
                self.altorendimiento()

        print("\nData loaded from .conf\n")

    def update_config_file(self, variable, value):
        config.read(utils.CONFIG_FILE)

        # We change our variable: config.set(section, variable, value)
        config.set('CONFIGURATION', str(variable), str(value))

        with open(utils.CONFIG_FILE, 'w') as configfile:
            config.write(configfile)

        print("Updated "+variable+" --> "+value+"\n")

    #Funcion para configuracion de bajo rendimiento
    def bajorendimiento(self, widgets=False):
        self.modo_actual="low"
        self.icono_actual=CURRENT_PATH+'/images/amd-1.png'
        self.testindicator.set_icon_full(CURRENT_PATH+'/images/amd-1.png', "Slimbook AMD")
            
        self.update_config_file("mode", self.modo_actual)
        proc = subprocess.Popen('pkexec slimbookamdcontroller-pkexec', shell=True)
        # proc.wait()
    #Funcion para configuracion de medio rendimiento
    def mediorendimiento(self, widgets=False):
        self.modo_actual="medium"
        self.icono_actual=CURRENT_PATH+'/images/amd-2.png'
        self.testindicator.set_icon_full(CURRENT_PATH+'/images/amd-2.png', "Slimbook AMD")

        self.update_config_file("mode", self.modo_actual)
        proc = subprocess.Popen('pkexec slimbookamdcontroller-pkexec', shell=True)
        # proc.wait()

    #Funcion para configuracion de alto rendimiento
    def altorendimiento(self, widgets=False):
        self.modo_actual="high"
        self.icono_actual=CURRENT_PATH+'/images/amd-3.png'
        self.testindicator.set_icon_full(CURRENT_PATH+'/images/amd-3.png', "Slimbook AMD")

        self.update_config_file("mode", self.modo_actual)
        proc = subprocess.Popen('pkexec slimbookamdcontroller-pkexec', shell=True)
        # proc.wait()


if __name__=="__main__":
    pid_name = "slimbook.amd.controller.indicator.pid"
    utils.application_lock(pid_name)
    
    if utils.get_secureboot_status():
        print("Error: SecureBoot is enabled", file=sys.stderr)
        sys.exit(1)

    app = Indicator()

    signal.signal(signal.SIGINT, signal.SIG_DFL)
    Gtk.main()
    
    utils.application_release(pid_name)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#This file just reads and applies ryzen configuaration info from .config
import os
import sys
import subprocess
import configparser
import utils
from pathlib import Path
from time import sleep

if (len(sys.argv)<2):
    sys.exit(127)
    
CONFIG_FILE = sys.argv[1]
if (not Path(CONFIG_FILE).exists()):
    print("Config file does not exists",file=sys.stderr)
    sys.exit(0)

print("Reading " + CONFIG_FILE, file=sys.stderr)
config = configparser.ConfigParser()
config.read(CONFIG_FILE)

modes = {"low":0, "medium":1, "high":2}

mode = config.get('CONFIGURATION', 'mode')
parameters = config.get('USER-CPU', 'cpu-parameters').split('/')

mode = modes[mode]

set_parameters = parameters[mode].split('-')
child = subprocess.Popen([
    "/usr/bin/ryzenadj",
    "--tctl-temp=95",
    "--slow-limit=" + set_parameters[0],
    "--stapm-limit=" + set_parameters[1],
    "--fast-limit=" + set_parameters[2]
    ])

child.wait()





import getpass
import gettext
import os
import subprocess
import locale

from pathlib import Path

CONFIG_FILE = str(Path.home()) + '/.config/slimbookamdcontroller/slimbookamdcontroller.conf'

DB_FILE = 'processors.db'

def get_user(from_file=None):
    try:
        user_name = getpass.getuser()
    except Exception:
        user_name = os.getlogin()

    if from_file and os.path.exists(from_file):
        exit_code, candidate = subprocess.getstatusoutput('cat {} | tail -n 1 | cut -f 2 -d "@"'.format(from_file))
        if exit_code != 0:
            user_name = candidate

    if user_name == 'root':
        if 'SUDO_USER' in os.environ and os.environ['SUDO_USER'] != 'root':
            user_name = os.environ['SUDO_USER']
        else:
            user_name = subprocess.getoutput('last -wn1 | head -n 1 | cut -f 1 -d " "')

    return user_name


def get_languages():
    languages = ['en']
    try:
        user_environ = locale.getlocale()[0]
        for lang in ["en", "es", "it", "fr"]:
            if user_environ.find(lang) >= 0:
                languages = [lang]
                break
    except Exception:
        pass
    return languages


def load_translation(filename):
    current_path = os.path.dirname(os.path.realpath(__file__))
    languages = get_languages()
    return gettext.translation(
        filename,
        os.path.join(current_path, 'translations'),
        languages=languages,
        fallback=True
    ).gettext

def get_cpu_info(var='info'):
    info = ()
    if var == 'name':
        return subprocess.getoutput('cat /proc/cpuinfo | grep name | uniq').split(':')[1].strip()
    if var == 'cores':
        return subprocess.getoutput('nproc')
    if var == 'threadspercore':
        cores = subprocess.getoutput('cat /proc/cpuinfo | grep "cpu cores" | uniq').split(':')[1].strip()
        return cores
    if var == 'info':
        print('Information get_cpu_info().\nOptions: \n\tname\n\tcores\n\tthreadspercore')

def get_secureboot_status():

    if (not Path("/sys/firmware/efi").exists()):
        return False
        
    SB_VAR = "/sys/firmware/efi/efivars/SecureBoot-8be4df61-93ca-11d2-aa0d-00e098032b8c"
    
    if (not Path(SB_VAR).exists()):
        return False
    
    sb = False
    f = open(SB_VAR,"rb")
    var = list(f.read())
    if (var[4] == 1):
        sb = True
    f.close()
    
    return sb

def get_run_dir():
    return "/run/user/{0}".format(os.getuid())
    
def is_pid_alive(pid):
    try:
        os.kill(pid,0)
        return True
    except:
        return False

def get_pid_from_file(name):
    run = get_run_dir()
    filename = Path(run + "/" + name)
    
    if (filename.exists()):
        f=open(str(filename),"r")
        data=f.readlines()
        f.close()
        if (len(data)>0):
            return int(data[0])
        else:
            return 0
    else:
        return 0
    
def create_pid_file(name):
    
    filename = get_run_dir() + "/" + name
    f=open(filename,"w")
    f.write("{0}".format(os.getpid()))
    f.close()
    
    return True

def destroy_pid_file(name):
    filename = get_run_dir() + "/" + name
    
    if (Path(filename).exists()):
        os.remove(filename)

def application_lock(name):
    pid = get_pid_from_file(name)

    if (pid>0):
        if (is_pid_alive(pid)):
            print("process is already running",file=sys.stderr)
            sys.exit(1)
        
    create_pid_file(name)
    
def application_release(name):
    destroy_pid_file(name)




import pyamdgpuinfo
import subprocess
import re

from constants.gpu_constants import (
    REGEX_MODEL_WITH_MODEL_IN_BRACKETS_MATCH,
    REGEX_MODEL_WITH_MODEL_IN_BRACKETS_FINDALL,
    REGEX_MODEL_ONLY_SERIES_NAME_MATCH,
    REGEX_MODEL_ONLY_SERIES_NAME_FINDALL,
    UNKNOWN_MODEL,
)

class GpuService:
    index: int

    def __init__(self, index: int):
        self.index = index

    @staticmethod
    def exists_amd_gpus() -> bool:
        return GpuService.get_number_of_gpus() > 0

    @staticmethod
    def get_number_of_gpus() -> int:
        return pyamdgpuinfo.detect_gpus()

    def get_model(self) -> str:
        model= pyamdgpuinfo.get_gpu(self.index).name
        if model is not None:
            return model
        else:
            slot = pyamdgpuinfo.get_gpu(self.index).pci_slot[5:]
            lspci_output = subprocess.getstatusoutput("lspci | grep -i "+slot+" | cut -d ':' -f3")[1].strip()

            try:
                if re.match(REGEX_MODEL_WITH_MODEL_IN_BRACKETS_MATCH, lspci_output, re.MULTILINE) is not None:
                    matches = re.findall(REGEX_MODEL_WITH_MODEL_IN_BRACKETS_FINDALL, lspci_output, re.MULTILINE)
                    model = matches[1].replace('[', '').replace(']', '')
                elif re.match(REGEX_MODEL_ONLY_SERIES_NAME_MATCH, lspci_output, re.MULTILINE) is not None:
                    model = re.findall(REGEX_MODEL_ONLY_SERIES_NAME_FINDALL, lspci_output)[1]
            except Exception as e:
                print(e)

            if model is None:
                model = UNKNOWN_MODEL

            return model.strip()

    def get_vram(self) -> str:
        vram_size = GpuService._get_vram_size(self)
        return '{} {}'.format(GpuService._convert_bytes_to_mbytes(self, vram_size), 'MB')

    def get_vram_usage(self) -> str:
        vram_usage = pyamdgpuinfo.get_gpu(self.index).query_vram_usage()
        return '{} {}'.format(GpuService._convert_bytes_to_mbytes(self, vram_usage), 'MB')

    def get_temp(self) -> str:
        return '{} ºC'.format(pyamdgpuinfo.get_gpu(self.index).query_temperature())

    def get_slot(self) -> str:
        return pyamdgpuinfo.get_gpu(self.index).pci_slot

    def get_slck(self) -> str:
        sclk = GpuService._get_freq(self, 'S')
        return '{} MHz'.format(sclk)

    def get_mlck(self) -> str:
        mclk = GpuService._get_freq(self, 'M')
        return '{} MHz'.format(mclk)

    def get_gpu_voltage(self) -> str:
        try:
            voltage = pyamdgpuinfo.get_gpu(self.index).query_graphics_voltage()
            return '{} V'.format(round(voltage, 3))
        except:
            return '0 V'

    def _get_vram_size(self) -> str:
        return pyamdgpuinfo.get_gpu(self.index).memory_info['vram_size']

    def _get_mem_usage_size(self) -> str:
        return pyamdgpuinfo.get_gpu(self.index).query_vram_usage()

    def _get_freq(self, freq_type: str):
        freq = 0;
        if freq_type == 'S':
            freq = pyamdgpuinfo.get_gpu(self.index).query_sclk()
        elif freq_type == 'M':
            freq = pyamdgpuinfo.get_gpu(self.index).query_mclk()

        return round(float(freq)/1000.0**2)

    def _convert_bytes_to_gbytes(self, vram_size):
        return round(float(vram_size)/1024.0**3)

    def _convert_bytes_to_mbytes(self, vram_size):
        return round(float(vram_size)/1024.0**2)



import math
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk, GLib
import utils
APPNAME = 'slimbookamdcontroller'
_ = utils.load_translation(APPNAME)

from matplotlib.backends.backend_gtk3agg import (
    FigureCanvasGTK3Agg as FigureCanvas)
from matplotlib.figure import Figure

from services.gpu_service import GpuService

from constants.gpu_constants import DYNAMIC_GPU_PROPERTIES, GPU_FREQ, GPU_VOLT, MEM_FREQ, MODEL, TEMP, VRAM, VRAM_USAGE

class GpuSection():
    def __init__(self, notebook):
        self.notebook = notebook
        self._optionSelected = 'temp'
        self._isOptionChanged = False
        self._x_array = []
        self._y_array = []
        self._max_values_by_gpu = {}

    def add(self):
        if GpuService.exists_amd_gpus():
            self.notebook = self.__add_gpus_pages()
            
        return self.notebook
        
    def __add_gpus_pages(self):
        number_of_gpus = GpuService.get_number_of_gpus()
        for gpu_index in range(number_of_gpus):
            page = Gtk.Box()
            page.set_border_width(10)
            page.set_halign(Gtk.Align.CENTER)
            box = self.__build_gpu_listbox(gpu_index)
            page.add(box)
            
            self.notebook.append_page(page, Gtk.Label(
                label="GPU {}".format(gpu_index)))
        return self.notebook

    def __update_label(self, label: Gtk.Label, serviceFunction):
        label.set_label(serviceFunction())
        return True


    def __build_gpu_listbox(self, gpu_index: int) -> Gtk.Grid:
        gpu = GpuService(gpu_index)

        GPU_INFO = {
            MODEL: gpu.get_model(),
            VRAM: gpu.get_vram(),
            VRAM_USAGE: gpu.get_vram_usage(),
            TEMP: gpu.get_temp(),
            GPU_FREQ: gpu.get_slck(),
            MEM_FREQ: gpu.get_mlck(),
            GPU_VOLT: gpu.get_gpu_voltage()
        }

        # Initialize the GPU max values
        self._max_values_by_gpu[gpu_index] = {
                'temp': 0,
                'gpuclk': 0,
                'memclk': 0,
                'gpuvolt': 0,
                'vramusage': 0
            }

        grid = Gtk.Grid(column_homogeneous=True, column_spacing=6, row_spacing=12)

        for index, key in enumerate(GPU_INFO):
            hbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=1)
            hbox_context = hbox.get_style_context()
            hbox_context.add_class('box-gpu-property')

            hbox.pack_start(Gtk.Label(label=_(key), xalign=0), True, True, 1)

            label = Gtk.Label(label=GPU_INFO[key], xalign=0)

            if self.__is_dynamic_property(key):
                serviceFunction = None
                if key == TEMP:
                    serviceFunction = gpu.get_temp
                elif key == GPU_FREQ:
                    serviceFunction = gpu.get_slck
                elif key == MEM_FREQ:
                    serviceFunction = gpu.get_mlck
                elif key == GPU_VOLT:
                    serviceFunction = gpu.get_gpu_voltage
                else:
                    serviceFunction = gpu.get_vram_usage
                GLib.timeout_add_seconds(2, self.__update_label, label, serviceFunction)

            hbox.pack_start(label, expand = True, fill = True, padding = 1)
            grid.attach(hbox, 0, index, 1, 1)

        grid.attach(self.__radio_buttons(), 1, 0, 1, 1)

        canvas = self.__render_chart(gpu)
        box_canvas = Gtk.Box()
        box_canvas.add(canvas)
        grid.attach(box_canvas, 1, 1, 1, 6)

        return grid

    def __radio_buttons(self):
        radio_buttons_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
        radio_buttons_box.set_halign(Gtk.Align.CENTER)

        button1 = Gtk.RadioButton.new_with_label_from_widget(None, "ºC")
        button1.connect("toggled", self.on_button_toggled, "temp")
        radio_buttons_box.pack_start(button1, False, False, 0)

        button2 = Gtk.RadioButton.new_with_mnemonic_from_widget(button1, "GPU")
        button2.connect("toggled", self.on_button_toggled, "gpuclk")
        radio_buttons_box.pack_start(button2, False, False, 0)

        button3 = Gtk.RadioButton.new_with_mnemonic_from_widget(button1, "Mem")
        button3.connect("toggled", self.on_button_toggled, "memclk")
        radio_buttons_box.pack_start(button3, False, False, 0)

        button4 = Gtk.RadioButton.new_with_mnemonic_from_widget(button1, "Volt")
        button4.connect("toggled", self.on_button_toggled, "gpuvolt")
        radio_buttons_box.pack_start(button4, False, False, 0)

        button5 = Gtk.RadioButton.new_with_mnemonic_from_widget(button1, "RAM")
        button5.connect("toggled", self.on_button_toggled, "vramusage")
        radio_buttons_box.pack_start(button5, False, False, 0)

        return radio_buttons_box

    def __is_dynamic_property(self, key):
        if key in DYNAMIC_GPU_PROPERTIES:
            return True
        return False

    def on_button_toggled(self, button, name):
        self._optionSelected = name
        self._isOptionChanged = True

    def __render_chart(self, gpu: GpuService) -> FigureCanvas:
        fig = Figure(figsize=(5, 4), dpi=100, )
        fig.patch.set_alpha(0.0)
        fig.set_tight_layout({"pad": .0})

        ax = fig.add_subplot()
        self.__set_ax_style(ax, gpu.index)

        GLib.timeout_add_seconds(1, self.__update_plot, gpu, ax)

        canvas = FigureCanvas(fig)
        canvas.set_size_request(400, 200)
        GLib.timeout_add_seconds(1, self.__drawCanvas, canvas )
        return canvas

    def __set_ax_style(self, ax, gpu_index):
        ax.set_facecolor('blue')
        ax.patch.set_alpha(0.1)
        ax.tick_params(axis='x', colors='white')
        ax.tick_params(axis='y', colors='white')
        ax.set_xticklabels([])
        ax.set_xticks([])
        ax.set_ylim(bottom=0)
        top=self.__get_max_value(gpu_index)
        if top > 0:
            ax.set_ylim(top=top)
        return True

    def  __update_plot(self, gpu: GpuService, ax):
        if self._isOptionChanged:
            self._isOptionChanged = False
            self._x_array = []
            self._y_array = []

        if len(self._x_array) > 0:
            self._x_array.append(self._x_array[-1] + 1)
        else:
            self._x_array.append(0)

        if self._optionSelected == "temp":
            self._y_array.append( float(gpu.get_temp().split(' ºC')[0]) )
        elif self._optionSelected == "gpuclk":
            self._y_array.append( float(gpu.get_slck().split(' MHz')[0]) )
        elif self._optionSelected == "memclk":
            self._y_array.append( float(gpu.get_mlck().split(' MHz')[0]) )
        elif self._optionSelected == "gpuvolt":
            self._y_array.append( float(gpu.get_gpu_voltage().split(' V')[0]) )
        elif self._optionSelected == "vramusage":
            self._y_array.append( float(gpu.get_vram_usage().split(' MB')[0]) )

        self.__clean_old_array_values()
        ax.cla()
        self.__set_ax_style(ax, gpu.index)
        ax.plot(self._x_array, self._y_array, '#641515')
        return True

    def __clean_old_array_values(self):
        if len(self._x_array) > 7 and len(self._y_array) > 7:
            self._x_array = self._x_array[1:]
            self._y_array = self._y_array[1:]
        return

    def __get_max_value(self, gpu_index):
        value = 0
        if len(self._y_array) > 0:
            #value = np.max(self._y_array)
            value = self._y_array[0]
            for y in self._y_array:
                if y > value:
                    value = y
            
            if self._optionSelected == "temp":
                value += 10
            elif self._optionSelected == "gpuvolt":
                value += 0.1
            else:
                value += 200

        max_values = self._max_values_by_gpu[gpu_index]
        if max_values[self._optionSelected] < value:
            max_values[self._optionSelected] = value
        else:
            value = max_values[self._optionSelected]
        
        return math.ceil(value)

    def __drawCanvas(self, canvas):
        canvas.draw()
        return True
